#!/usr/bin/env python3
import time

from resemble.lib import symbiosis, fractal

from resemble import frwrap, engine

from resemble import debug

world = None
worldrules = None
scanrules = None
age = 1
last_position = None
last_angle = 0
last_hz_scale = 1
last_vt_scale = 1

ZOOM = 0

LAST = 0
SPEED = 0.5

def resize(w, h):
  fractal.resize(w, h)

def render():
  global world, worldrules, scanrules, age, last_position, last_angle
  global last_hz_scale, last_vt_scale, LAST
  now = time.time()
  if now - LAST > SPEED:
    age = 0
    last_position = engine.get_position("player")
    engine.step(worldrules, scanrules, world)
    position = engine.get_position("player")
    if last_position:
      # TODO: Sort these out...
      if position.lineage == last_position.lineage + 1:
        last_angle = engine.dirint(position.parent.fraxel.orientation) * 90
        last_hz_scale = 1/position.surface.hz_scale
        last_vt_scale = 1/position.surface.vt_scale
      elif position.lineage == last_position.lineage - 1:
        last_angle = engine.dirint(last_position.parent.fraxel.orientation) * 90
        last_hz_scale = last_position.surface.hz_scale
        last_vt_scale = last_position.surface.vt_scale
      elif position.lineage == last_position.lineage\
      and position.surface != last_position.surface:
        last_angle = (engine.dirint(last_position.parent.fraxel.orientation) -\
                      engine.dirint(position.parent.fraxel.orientation)) * 90
        last_hz_scale = last_position.surface.hz_scale /\
                        float(position.surface.hz_scale)
        last_vt_scale = last_position.surface.vt_scale /\
                        float(position.surface.vt_scale)
      # TODO: Are there any other cases to address?
      else:
        last_angle = 0
        last_hz_scale = 1
        last_vt_scale = 1
    LAST = now
  age += 0.01 # TODO: adjust this.
  symbiosis.clear_color_buffer()
  position = engine.get_position("player")
  position.ensure_heritage()
  fractal.render(
    position,
    float(last_angle),
    float(last_hz_scale),
    float(last_vt_scale),
    float(age)
  )
  return True

def keyboard(pushed, key, x, y):
  global ZOOM
  if not pushed:
    return
  debug.debug("input", "Key pressed: '{}'".format(key.decode()))
  if key == b'q':
    symbiosis.quit()
  elif key == b'k':
    ZOOM += 0.2
    fractal.set_zoom(ZOOM)
    ZOOM = fractal.get_zoom()
  elif key == b'j':
    ZOOM -= 0.2
    fractal.set_zoom(ZOOM)
    ZOOM = fractal.get_zoom()
  elif key == b'0':
    ZOOM = 0
    fractal.set_zoom(ZOOM)
    ZOOM = fractal.get_zoom()
  elif key == b'w':
    engine.raise_signals("move_north")
  elif key == b'a':
    engine.raise_signals("move_west")
  elif key == b's':
    engine.raise_signals("move_south")
  elif key == b'd':
    engine.raise_signals("move_east")

callbacks = {
  "render": render,
  "resize": resize,
  "keyboard": keyboard,
}

def define_test_world():
  global world
  fractal.set_zoom(ZOOM)
  world = frwrap.load_world("example")

  for name in world:
    debug.debug("world", world[name])

  engine.set_position("player", frwrap.Position(1, 1, world["first"]))

def define_test_rules():
  global worldrules, scanrules
  worldrules, scanrules = engine.load_rules("example")

if __name__ == "__main__":
  define_test_world()
  define_test_rules()
  symbiosis.prepare(
    width=800,
    height=600,
    name="Toponaut",
    background=(1.0, 0, 0, 1.0),
  )
  symbiosis.attach(callbacks)
  symbiosis.loop()
