#!/usr/bin/env python3
import math

from lib import symbiosis, fractal

position = None
surface = None
all_surfaces = []

ZOOM = 0

def hallucinate_parent(position):
  for s in all_surfaces:
    for x in range(s.sc_width * s.hz_scale):
      for y in range(s.sc_height * s.vt_scale):
        if s.get(x, y)[0] == position.surface:
          return fractal.Position(x, y, s)
  return None

def resize(w, h):
  fractal.resize(w, h)

def render():
  global position
  symbiosis.clear_color_buffer()
  size_required = fractal.get_view_size()
  size_x = 1
  size_y = 1
  fixing = position
  while size_x < size_required or size_y < size_required:
    if fixing.parent == None:
      fixing.parent = hallucinate_parent(fixing)
    if fixing.parent:
      size_x *= fixing.surface.hz_scale
      size_y *= fixing.surface.vt_scale
      fixing = fixing.parent
    else:
      break
  fractal.render(position)
  return True

def keyboard(key, x, y):
  global ZOOM
  print("Key pressed: '{}'".format(key.decode()))
  if key == b'q':
    symbiosis.quit()
  if key == b'k':
    ZOOM += 0.2
    fractal.set_zoom(ZOOM)
    ZOOM = fractal.get_zoom()
  if key == b'j':
    ZOOM -= 0.2
    fractal.set_zoom(ZOOM)
    ZOOM = fractal.get_zoom()
  if key == b'0':
    ZOOM = 0
    fractal.set_zoom(ZOOM)
    ZOOM = fractal.get_zoom()

callbacks = {
  "render": render,
  "resize": resize,
  "keyboard": keyboard,
}

def define_test_world():
  global position, surface
  fractal.set_zoom(ZOOM)
  surface = fractal.Surface(3, 5, 1, 1)
  for x in range(surface.hz_scale * surface.sc_width):
    for y in range(surface.vt_scale * surface.sc_height):
      surface.set(x, y, (255, 255, 255))
  for x in range(1, surface.hz_scale * surface.sc_width - 1):
    for y in range(surface.vt_scale * surface.sc_height - 1):
      surface.set(x, y, (0, 0, 0))
  surface.set(0, 0, (0, 0, 0))
  surface.set(-1, 0, (0, 0, 0))
  surface.set(1, -2, surface, 1)
  surface.set(1, -1, (93, 126, 230))

  all_surfaces.append(surface)

  position = fractal.Position(1, 3, surface)

  for y in range(surface.vt_scale * surface.sc_height):
    for x in range(surface.hz_scale * surface.sc_width):
      print(surface.get(x, y), end=" ")
    print()

if __name__ == "__main__":
  define_test_world()
  symbiosis.prepare(
    width=800,
    height=600,
    name="Toponaut",
    background=(1.0, 0, 0, 1.0),
  )
  symbiosis.attach(callbacks)
  symbiosis.loop()
