#!/usr/bin/env python3
from .lib import symbiosis
from .lib import fractal
from . import frwrap

position = None
surface = None
world = {}

ZOOM = 0

def resize(w, h):
  fractal.resize(w, h)

def render():
  global position
  symbiosis.clear_color_buffer()
  position.ensure_heritage()
  fractal.render(position)
  return True

def keyboard(key, x, y):
  global ZOOM
  print("Key pressed: '{}'".format(key.decode()))
  if key == b'q':
    symbiosis.quit()
  if key == b'k':
    ZOOM += 0.2
    fractal.set_zoom(ZOOM)
    ZOOM = fractal.get_zoom()
  if key == b'j':
    ZOOM -= 0.2
    fractal.set_zoom(ZOOM)
    ZOOM = fractal.get_zoom()
  if key == b'0':
    ZOOM = 0
    fractal.set_zoom(ZOOM)
    ZOOM = fractal.get_zoom()

callbacks = {
  "render": render,
  "resize": resize,
  "keyboard": keyboard,
}

def define_test_world():
  global position, surface
  fractal.set_zoom(ZOOM)
  surface = frwrap.Surface("test", world, 3, 5, 1, 1)
  surface.add_border(0, 0, 1)
  for x in range(surface.width):
    surface.set(x, 0, (0, 0, 0))
  surface.set(1, -2, surface, 'e')
  surface.set(1, -1, (0.62, 0.6, 0.95))

  position = frwrap.Position(1, 3, surface)

  for y in range(surface.height):
    for x in range(surface.width):
      print(surface.get(x, y), end=" ")
    print()

if __name__ == "__main__":
  define_test_world()
  symbiosis.prepare(
    width=800,
    height=600,
    name="Toponaut",
    background=(1.0, 0, 0, 1.0),
  )
  symbiosis.attach(callbacks)
  symbiosis.loop()
